#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.namibia import Namibia
from tests.common import CommonCountryTests


class TestNamibia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1991, 2050)
        super().setUpClass(Namibia, years=years, years_non_observed=years)

    def test_special_holidays(self):
        self.assertHoliday(
            "1994-03-01",
            "1994-12-07",
            "1995-10-24",
            "1997-09-26",
            "1998-11-30",
            "1999-11-30",
            "1999-12-31",
            "2000-01-03",
            "2014-11-28",
            "2015-11-27",
            "2019-11-27",
            "2020-11-25",
            "2024-02-25",
            "2024-11-27",
            "2025-03-01",
        )

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1991, 2050)))
        obs_dt = (
            "1995-01-02",
            "2006-01-02",
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-03-21" for year in range(1991, 2050)))
        obs_dt = (
            "1993-03-22",
            "1999-03-22",
            "2004-03-22",
            "2010-03-22",
            "2021-03-22",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(1991, 2050))

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(1991, 2050))

    def test_workers_day(self):
        name = "Workers' Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1991, 2050)))
        obs_dt = (
            "1994-05-02",
            "2005-05-02",
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_cassinga_day(self):
        name = "Cassinga Day"
        self.assertHolidayName(name, (f"{year}-05-04" for year in range(1991, 2050)))
        obs_dt = (
            "1997-05-05",
            "2003-05-05",
            "2008-05-05",
            "2014-05-05",
            "2025-05-05",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_africa_day(self):
        name = "Africa Day"
        self.assertHolidayName(name, (f"{year}-05-25" for year in range(1991, 2050)))
        obs_dt = (
            "1997-05-26",
            "2003-05-26",
            "2008-05-26",
            "2014-05-26",
            "2025-05-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_genocide_remembrance_day(self):
        name = "Genocide Remembrance Day"
        self.assertHolidayName(name, (f"{year}-05-28" for year in range(2025, 2050)))
        self.assertNoHolidayName(name, range(1991, 2025))

    def test_ascension_day(self):
        name = "Ascension Day"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, range(1991, 2050))

    def test_heroes_day(self):
        name = "Heroes' Day"
        self.assertHolidayName(name, (f"{year}-08-26" for year in range(1991, 2050)))
        obs_dt = (
            "2001-08-27",
            "2007-08-27",
            "2012-08-27",
            "2018-08-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_namibian_women_and_international_human_rights_day(self):
        name_1 = "International Human Rights Day"
        self.assertHolidayName(name_1, (f"{year}-09-10" for year in range(1991, 2005)))
        self.assertNoHolidayName(name_1, range(2005, 2050))
        obs_dt = (
            "1995-09-11",
            "2000-09-11",
        )
        self.assertHolidayName(f"{name_1} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

        name_2 = "Day of the Namibian Women and International Human Rights Day"
        self.assertHolidayName(name_2, (f"{year}-09-10" for year in range(2005, 2050)))
        self.assertNoHolidayName(name_2, range(1991, 2005))
        obs_dt = (
            "2006-09-11",
            "2017-09-11",
            "2023-09-11",
        )
        self.assertHolidayName(f"{name_2} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in range(1991, 2050)))

    def test_family_day(self):
        name = "Family Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in range(1991, 2050)))
        obs_dt = (
            "1993-12-27",
            "1999-12-27",
            "2004-12-27",
            "2010-12-27",
            "2021-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-02-25", "Burial ceremony of Dr. Hage Gottfried Geingob"),
            ("2024-03-21", "Independence Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-01", "Workers' Day"),
            ("2024-05-04", "Cassinga Day"),
            ("2024-05-09", "Ascension Day"),
            ("2024-05-25", "Africa Day"),
            ("2024-08-26", "Heroes' Day"),
            ("2024-09-10", "Day of the Namibian Women and International Human Rights Day"),
            ("2024-11-27", "General Election Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Family Day"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-03-01", "Burial ceremony of Dr. Sam Shafiishuna Nujoma"),
            ("2025-03-21", "Independence Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Workers' Day"),
            ("2025-05-04", "Cassinga Day"),
            ("2025-05-05", "Cassinga Day (observed)"),
            ("2025-05-25", "Africa Day"),
            ("2025-05-26", "Africa Day (observed)"),
            ("2025-05-28", "Genocide Remembrance Day"),
            ("2025-05-29", "Ascension Day"),
            ("2025-08-26", "Heroes' Day"),
            ("2025-09-10", "Day of the Namibian Women and International Human Rights Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Family Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-03-01", "Burial ceremony of Dr. Sam Shafiishuna Nujoma"),
            ("2025-03-21", "Independence Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Workers' Day"),
            ("2025-05-04", "Cassinga Day"),
            ("2025-05-05", "Cassinga Day (observed)"),
            ("2025-05-25", "Africa Day"),
            ("2025-05-26", "Africa Day (observed)"),
            ("2025-05-28", "Genocide Remembrance Day"),
            ("2025-05-29", "Ascension Day"),
            ("2025-08-26", "Heroes' Day"),
            ("2025-09-10", "Day of the Namibian Women and International Human Rights Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Family Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2025-01-01", "Новий рік"),
            ("2025-03-01", "Церемонія поховання Сема Шафіішуна Нуйоми"),
            ("2025-03-21", "День незалежності"),
            ("2025-04-18", "Страсна пʼятниця"),
            ("2025-04-21", "Великодній понеділок"),
            ("2025-05-01", "День трудящих"),
            ("2025-05-04", "День Кассінги"),
            ("2025-05-05", "День Кассінги (вихідний)"),
            ("2025-05-25", "День Африки"),
            ("2025-05-26", "День Африки (вихідний)"),
            ("2025-05-28", "День памʼяті жертв геноциду"),
            ("2025-05-29", "Вознесіння Господнє"),
            ("2025-08-26", "День Героїв"),
            ("2025-09-10", "День намібійських жінок та Міжнародний день прав людини"),
            ("2025-12-25", "Різдво Христове"),
            ("2025-12-26", "День родини"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import (
    JAN,
    FEB,
    MAR,
    APR,
    MAY,
    JUN,
    JUL,
    AUG,
    SEP,
    OCT,
    NOV,
    DEC,
    FRI,
)
from holidays.groups import IslamicHolidays, PersianCalendarHolidays, StaticHolidays
from holidays.holiday_base import HolidayBase


class Iran(HolidayBase, IslamicHolidays, PersianCalendarHolidays, StaticHolidays):
    """Iran holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Iran>
        * <https://fa.wikipedia.org/wiki/تعطیلات_عمومی_در_ایران>
        * <https://web.archive.org/web/20250426102648/https://www.time.ir/>
        * <https://web.archive.org/web/20240804012557/https://www.chicagotribune.com/1989/06/07/mourners-rip-shroud-khomeinis-body-falls/>
        * <https://web.archive.org/web/20170222200759/http://www.hvm.ir/LawDetailNews.aspx?id=9017>
        * <https://en.wikipedia.org/wiki/Workweek_and_weekend>
    """

    country = "IR"
    default_language = "fa_IR"
    # %s (estimated).
    estimated_label = tr("%s (تخمینی)")
    supported_languages = ("en_US", "fa_IR")
    start_year = 1980
    weekend = {FRI}

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.

        In Iran, the dates of the Islamic calendar usually fall a day later than
        the corresponding dates in the Umm al-Qura calendar.
        """
        IslamicHolidays.__init__(
            self,
            cls=IranIslamicHolidays,
            show_estimated=islamic_show_estimated,
            calendar_delta_days=+1,
        )
        PersianCalendarHolidays.__init__(self)
        StaticHolidays.__init__(self, IranStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Persian calendar holidays.

        # Islamic Revolution Day.
        self._add_islamic_revolution_day(tr("پیروزی انقلاب اسلامی"))

        # Iranian Oil Industry Nationalization Day.
        self._add_oil_nationalization_day(tr("روز ملی شدن صنعت نفت ایران"))

        # Last Day of Year.
        self._add_last_day_of_year(tr("آخرین روز سال"))

        # Nowruz.
        self._add_nowruz_day(tr("جشن نوروز"))

        # Nowruz Holiday.
        name = tr("عیدنوروز")
        self._add_nowruz_day_two(name)
        self._add_nowruz_day_three(name)
        self._add_nowruz_day_four(name)

        # Islamic Republic Day.
        self._add_islamic_republic_day(tr("روز جمهوری اسلامی"))

        # Nature's Day.
        self._add_natures_day(tr("روز طبیعت"))

        # Immediate holiday in 1989, and subsequently codified from 1990 onwards.
        if self._year >= 1989:
            # Death of Imam Khomeini.
            self._add_death_of_khomeini_day(tr("رحلت حضرت امام خمینی"))

        # 15 Khordad Uprising.
        self._add_khordad_uprising_day(tr("قیام 15 خرداد"))

        # Islamic holidays.

        # Tasua.
        self._add_tasua_day(tr("تاسوعای حسینی"))

        # Ashura.
        self._add_ashura_day(tr("عاشورای حسینی"))

        # Arbaeen.
        self._add_arbaeen_day(tr("اربعین حسینی"))

        # Death of Prophet Muhammad and Martyrdom of Hasan ibn Ali.
        self._add_prophet_death_day(tr("رحلت رسول اکرم؛شهادت امام حسن مجتبی علیه السلام"))

        # Martyrdom of Ali al-Rida.
        self._add_ali_al_rida_death_day(tr("شهادت امام رضا علیه السلام"))

        # Martyrdom of Hasan al-Askari.
        self._add_hasan_al_askari_death_day(tr("شهادت امام حسن عسکری علیه السلام"))

        # Birthday of Muhammad and Imam Ja'far al-Sadiq.
        self._add_sadiq_birthday_day(tr("میلاد رسول اکرم و امام جعفر صادق علیه السلام"))

        # Martyrdom of Fatima.
        self._add_fatima_death_day(tr("شهادت حضرت فاطمه زهرا سلام الله علیها"))

        # Birthday of Imam Ali.
        self._add_ali_birthday_day(tr("ولادت امام علی علیه السلام و روز پدر"))

        # Isra' and Mi'raj.
        self._add_isra_and_miraj_day(tr("مبعث رسول اکرم (ص)"))

        self._add_imam_mahdi_birthday_day(
            # Birthday of Mahdi.
            tr("ولادت حضرت قائم عجل الله تعالی فرجه و جشن نیمه شعبان")
        )

        # Martyrdom of Imam Ali.
        self._add_ali_death_day(tr("شهادت حضرت علی علیه السلام"))

        # Eid al-Fitr.
        self._add_eid_al_fitr_day(tr("عید سعید فطر"))

        # Eid al-Fitr Holiday.
        self._add_eid_al_fitr_day_two(tr("تعطیل به مناسبت عید سعید فطر"))

        # Martyrdom of Imam Ja'far al-Sadiq.
        self._add_sadiq_death_day(tr("شهادت امام جعفر صادق علیه السلام"))

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("عید سعید قربان"))

        # Eid al-Ghadeer.
        self._add_eid_al_ghadir_day(tr("عید سعید غدیر خم"))


class IR(Iran):
    pass


class IRN(Iran):
    pass


class IranIslamicHolidays(_CustomIslamicHolidays):
    ALI_AL_RIDA_DEATH_DATES_CONFIRMED_YEARS = (2001, 2025)
    ALI_AL_RIDA_DEATH_DATES = {
        2005: (APR, 9),
        2008: (MAR, 8),
        2013: (JAN, 12),
        2018: (NOV, 8),
        2020: (OCT, 17),
    }

    ALI_BIRTHDAY_DATES_CONFIRMED_YEARS = (2001, 2025)
    ALI_BIRTHDAY_DATES = {
        2003: (SEP, 10),
        2008: (JUL, 16),
        2009: (JUL, 6),
        2015: (MAY, 2),
        2019: (MAR, 20),
        2020: (MAR, 8),
        2021: (FEB, 25),
        2023: (FEB, 4),
        2024: (JAN, 25),
    }

    ALI_DEATH_DATES_CONFIRMED_YEARS = (2001, 2025)
    ALI_DEATH_DATES = {
        2007: (OCT, 3),
        2009: (SEP, 11),
        2011: (AUG, 21),
        2015: (JUL, 8),
        2017: (JUN, 16),
        2023: (APR, 12),
    }

    ARBAEEN_DATES_CONFIRMED_YEARS = (2001, 2025)
    ARBAEEN_DATES = {
        2001: (MAY, 14),
        2002: (MAY, 3),
        2007: (MAR, 10),
        2012: (JAN, 14),
        2013: ((JAN, 3), (DEC, 23)),
        2015: (DEC, 2),
        2016: (NOV, 20),
        2017: (NOV, 9),
        2019: (OCT, 19),
        2021: (SEP, 27),
        2025: (AUG, 14),
    }

    ASHURA_DATES_CONFIRMED_YEARS = (2001, 2025)
    ASHURA_DATES = {
        2006: (FEB, 9),
        2008: (JAN, 19),
        2009: ((JAN, 7), (DEC, 27)),
        2010: (DEC, 16),
        2018: (SEP, 20),
        2022: (AUG, 8),
        2023: (JUL, 28),
        2024: (JUL, 16),
    }

    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2001, 2025)
    EID_AL_ADHA_DATES = {
        2005: (JAN, 21),
        2006: ((JAN, 11), (DEC, 31)),
        2012: (OCT, 26),
        2017: (SEP, 1),
        2020: (JUL, 31),
        2025: (JUN, 6),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2001, 2025)
    EID_AL_FITR_DATES = {
        2001: (DEC, 16),
        2004: (NOV, 14),
        2007: (OCT, 13),
        2008: (OCT, 1),
        2009: (SEP, 20),
        2010: (SEP, 10),
        2012: (AUG, 19),
        2016: (JUL, 6),
        2018: (JUN, 15),
        2020: (MAY, 24),
        2021: (MAY, 13),
        2024: (APR, 10),
    }

    EID_AL_GHADIR_DATES_CONFIRMED_YEARS = (2001, 2025)
    EID_AL_GHADIR_DATES = {
        2005: (JAN, 29),
        2007: ((JAN, 8), (DEC, 29)),
        2012: (NOV, 3),
        2017: (SEP, 9),
        2020: (AUG, 8),
        2025: (JUN, 14),
    }

    FATIMA_DEATH_DATES_CONFIRMED_YEARS = (2001, 2025)
    FATIMA_DEATH_DATES = {
        2002: (AUG, 12),
        2006: (JUN, 29),
        2007: (JUN, 18),
        2008: (JUN, 7),
        2010: (MAY, 17),
        2014: (APR, 3),
        2017: (MAR, 2),
        2022: ((JAN, 6), (DEC, 27)),
        2025: (NOV, 24),
    }

    HASAN_AL_ASKARI_DEATH_DATES_CONFIRMED_YEARS = (2001, 2025)
    HASAN_AL_ASKARI_DEATH_DATES = {
        2005: (APR, 17),
        2008: (MAR, 16),
        2013: (JAN, 20),
        2018: (NOV, 16),
        2020: (OCT, 25),
    }

    IMAM_MAHDI_BIRTHDAY_DATES_CONFIRMED_YEARS = (2001, 2025)
    IMAM_MAHDI_BIRTHDAY_DATES = {
        2004: (OCT, 1),
        2010: (JUL, 27),
        2012: (JUL, 5),
        2013: (JUN, 24),
        2014: (JUN, 13),
        2016: (MAY, 22),
        2022: (MAR, 18),
        2024: (FEB, 25),
        2025: (FEB, 14),
    }

    ISRA_AND_MIRAJ_DATES_CONFIRMED_YEARS = (2001, 2025)
    ISRA_AND_MIRAJ_DATES = {
        2003: (SEP, 24),
        2008: (JUL, 30),
        2009: (JUL, 20),
        2015: (MAY, 16),
        2019: (APR, 3),
        2020: (MAR, 22),
        2021: (MAR, 11),
        2023: (FEB, 18),
        2024: (FEB, 8),
    }

    PROPHET_DEATH_DATES_CONFIRMED_YEARS = (2001, 2025)
    PROPHET_DEATH_DATES = {
        2001: (MAY, 22),
        2002: (MAY, 11),
        2007: (MAR, 18),
        2012: (JAN, 22),
        2013: ((JAN, 11), (DEC, 31)),
        2015: (DEC, 10),
        2016: (NOV, 28),
        2017: (NOV, 17),
        2019: (OCT, 27),
        2021: (OCT, 5),
        2025: (AUG, 22),
    }

    SADIQ_BIRTHDAY_DATES_CONFIRMED_YEARS = (2001, 2025)
    SADIQ_BIRTHDAY_DATES = {
        2005: (APR, 26),
        2008: (MAR, 25),
        2013: (JAN, 29),
        2018: (NOV, 25),
        2020: (NOV, 3),
    }

    SADIQ_DEATH_DATES_CONFIRMED_YEARS = (2001, 2025)
    SADIQ_DEATH_DATES = {
        2001: (JAN, 20),
        2002: ((JAN, 9), (DEC, 30)),
        2004: (DEC, 8),
        2007: (NOV, 6),
        2008: (OCT, 25),
        2009: (OCT, 14),
        2010: (OCT, 4),
        2012: (SEP, 12),
        2016: (JUL, 30),
        2018: (JUL, 9),
        2020: (JUN, 17),
        2021: (JUN, 6),
        2024: (MAY, 4),
    }

    TASUA_DATES_CONFIRMED_YEARS = (2001, 2025)
    TASUA_DATES = {
        2006: (FEB, 8),
        2008: (JAN, 18),
        2009: ((JAN, 6), (DEC, 26)),
        2010: (DEC, 15),
        2018: (SEP, 19),
        2022: (AUG, 7),
        2023: (JUL, 27),
        2024: (JUL, 15),
    }


class IranStaticHolidays:
    """Iran special holidays.

    References:
       * <http://archive.today/2026.03.02-035111/https://www.dw.com/en/iran-unprecedented-heat-prompts-two-day-public-holiday/a-66410894>
       * <http://archive.today/2026.03.02-035349/https://abanonline.com/political/government/2026-03-01/هییت-دولت-40-روز-عزای-عمومی-7-روز-تعطیل-عمو/>
       * <http://archive.today/2026.03.02-035353/https://www.tasnimnews.ir/en/news/2026/03/01/3528137/iran-declares-40-days-of-national-mourning-after-leader-s-martyrdom-after-leader-s-martyrdom>
    """

    # Public Holiday.
    public_holiday = tr("تعطیلی عمومی")

    special_public_holidays = {
        2023: (
            (AUG, 1, public_holiday),
            (AUG, 2, public_holiday),
        ),
        2026: (
            (MAR, 1, public_holiday),
            (MAR, 2, public_holiday),
            (MAR, 3, public_holiday),
            (MAR, 4, public_holiday),
            (MAR, 5, public_holiday),
            (MAR, 6, public_holiday),
            (MAR, 7, public_holiday),
        ),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.malaysia import Malaysia
from tests.common import CommonCountryTests


class TestMalaysia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Malaysia, with_subdiv_special_flags=True)

    def test_2023(self):
        rows = (
            ("2023-01-01", (0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1)),
            ("2023-01-02", (0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1)),
            ("2023-01-14", (0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-01-22", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-01-23", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-01-24", (0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1)),
            ("2023-02-01", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1)),
            ("2023-02-05", (1, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1)),
            ("2023-02-06", (0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1)),
            ("2023-02-18", (0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0)),
            ("2023-02-19", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0)),
            ("2023-03-04", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0)),
            ("2023-03-05", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0)),
            ("2023-03-23", (1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-04-07", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0)),
            ("2023-04-08", (0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1)),
            ("2023-04-09", (0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0)),
            ("2023-04-15", (0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-04-21", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-04-22", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-04-23", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-04-24", (0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-04-26", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0)),
            ("2023-05-01", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-05-04", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-05-17", (0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-05-22", (0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-05-30", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0)),
            ("2023-05-31", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0)),
            ("2023-06-01", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0)),
            ("2023-06-02", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0)),
            ("2023-06-05", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-06-18", (0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-06-28", (0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0)),
            ("2023-06-29", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-06-30", (0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0)),
            ("2023-07-02", (0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-07-07", (0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-07-08", (0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-07-19", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-07-22", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0)),
            ("2023-07-30", (0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-07-31", (0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-08-23", (1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-08-24", (0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-08-31", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-09-16", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-09-17", (0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0)),
            ("2023-09-28", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
            ("2023-09-29", (0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-09-30", (0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-10-07", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0)),
            ("2023-10-14", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0)),
            ("2023-11-03", (0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0)),
            ("2023-11-12", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1)),
            ("2023-11-13", (0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1)),
            ("2023-12-11", (0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0)),
            ("2023-12-24", (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0)),
            ("2023-12-25", (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)),
        )

        for col, subdiv in enumerate(Malaysia.subdivisions):
            subdiv_holidays = self.subdiv_holidays[subdiv]
            for dt, is_holiday in rows:
                self.assertEqual(dt in subdiv_holidays, is_holiday[col])

    def test_special_holidays(self):
        self.assertHoliday(
            "1999-11-29",
            "2017-04-24",
            "2017-09-04",
            "2018-05-09",
            "2019-07-30",
            "2022-11-18",
            "2022-11-19",
            "2022-11-28",
            "2023-04-21",
            "2025-09-15",
        )

    def test_special_subdiv_holidays(self):
        for subdiv in ("14", "15", "16"):
            self.assertHoliday(self.subdiv_holidays[subdiv], "2021-12-03")

        for subdiv in ("01", "02", "03", "11"):
            self.assertHoliday(
                self.subdiv_holidays[subdiv],
                "2018-05-10",
                "2018-05-13",
                "2022-05-04",
            )
            self.assertNoNonObservedHoliday(
                self.subdiv_holidays_non_observed[subdiv],
                "2022-05-04",
            )

        for subdiv in ("04", "05", "06", "07", "08", "09", "10", "14", "15", "16"):
            self.assertHoliday(self.subdiv_holidays[subdiv], "2018-05-10", "2018-05-11")

        self.assertHoliday(self.subdiv_holidays["13"], "2018-05-17", "2018-05-18")

        for subdiv in ("04", "05", "06", "07", "08", "09", "10", "11", "12", "14", "15", "16"):
            self.assertHoliday(self.subdiv_holidays[subdiv], "2007-01-02")
            self.assertNoNonObservedHoliday(
                self.subdiv_holidays_non_observed[subdiv],
                "2007-01-02",
            )

    def test_new_years_day(self):
        name = "Tahun Baharu"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"04", "05", "06", "07", "08", "10", "12", "13", "14", "15", "16"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-01" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_federal_territory_day(self):
        name = "Hari Wilayah Persekutuan"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"14", "15", "16"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-02-01" for year in range(1974, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1974))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_labor_day(self):
        name = "Hari Pekerja"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1973, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1973))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"12", "13"}:
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-04-10",
                    "2021-04-02",
                    "2022-04-15",
                    "2023-04-07",
                    "2024-03-29",
                    "2025-04-18",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_pesta_kaamatan(self):
        name = "Pesta Kaamatan"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"12", "15"}:
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-05-30" for year in self.full_range),
                    (f"{year}-05-31" for year in self.full_range),
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_yang_di_pertuan_agong_birthday(self):
        name = "Hari Keputeraan Rasmi Seri Paduka Baginda Yang di-Pertuan Agong"
        self.assertHolidayName(
            name,
            "2012-06-02",
            "2013-06-01",
            "2014-06-07",
            "2015-06-06",
            "2016-06-04",
            "2017-09-09",
            "2018-09-09",
            "2019-09-09",
            "2020-06-08",
            "2021-06-07",
            "2022-06-06",
            "2023-06-05",
            "2024-06-03",
            "2025-06-02",
        )
        self.assertHolidayName(name, self.full_range)

    def test_malaysia_day(self):
        name = "Hari Malaysia"
        self.assertHolidayName(name, (f"{year}-09-16" for year in range(2010, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2010))

    def test_deepavali(self):
        name = "Hari Deepavali"
        self.assertNoHolidayName(name)
        dt = (
            "2014-10-22",
            "2015-11-10",
            "2016-10-29",
            "2017-10-18",
            "2018-11-06",
            "2019-10-27",
            "2020-11-14",
            "2021-11-04",
            "2022-10-24",
            "2023-11-12",
            "2024-10-31",
            "2025-10-20",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                case "13":
                    self.assertNoHolidayName(name, holidays)
                case "15":
                    self.assertHolidayName(name, holidays, dt)
                    self.assertHolidayName(name, holidays, range(2014, self.end_year))
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 2014))
                case _:
                    self.assertHolidayName(name, holidays, dt)

    def test_thaipusam(self):
        name = "Hari Thaipusam"
        self.assertNoHolidayName(name)
        dt_1 = (
            "2011-01-20",
            "2012-02-07",
            "2013-01-27",
            "2014-01-17",
            "2015-02-03",
            "2016-01-24",
            "2017-02-09",
            "2018-01-31",
            "2019-01-21",
            "2020-02-08",
            "2021-01-28",
        )
        dt_2 = (
            "2022-01-18",
            "2023-02-05",
            "2024-01-25",
            "2025-02-11",
        )

        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                case "01" | "05" | "07" | "08" | "10" | "14" | "16":
                    self.assertHolidayName(name, holidays, dt_1, dt_2)
                    self.assertHolidayName(name, holidays, self.full_range)
                case "02":
                    self.assertHolidayName(name, holidays, dt_2)
                    self.assertHolidayName(name, holidays, range(2022, self.end_year))
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 2022))
                case _:
                    self.assertNoHolidayName(name, holidays)

    def test_islamic_new_year(self):
        name = "Awal Muharam"
        self.assertHolidayName(
            name,
            "2012-11-15",
            "2013-11-05",
            "2014-10-25",
            "2015-10-14",
            "2016-10-02",
            "2017-09-22",
            "2018-09-11",
            "2019-09-01",
            "2020-08-20",
            "2021-08-10",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-27",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(1995, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 1995))

    def test_prophets_birthday(self):
        name = "Hari Keputeraan Nabi Muhammad S.A.W."
        self.assertHolidayName(
            name,
            "2012-02-05",
            "2013-01-24",
            "2014-01-14",
            "2015-01-03",
            "2015-12-24",
            "2016-12-12",
            "2017-12-01",
            "2018-11-20",
            "2019-11-09",
            "2020-10-29",
            "2021-10-19",
            "2022-10-10",
            "2023-09-28",
            "2024-09-16",
            "2025-09-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr(self):
        name = "Hari Raya Puasa"
        self.assertHolidayName(
            name,
            "2012-08-19",
            "2013-08-08",
            "2014-07-28",
            "2015-07-17",
            "2016-07-06",
            "2017-06-25",
            "2018-06-15",
            "2019-06-05",
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-22",
            "2024-04-10",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr_second_day(self):
        name = "Hari Raya Puasa (Hari Kedua)"
        self.assertHolidayName(
            name,
            "2012-08-20",
            "2013-08-09",
            "2014-07-29",
            "2015-07-18",
            "2016-07-07",
            "2017-06-26",
            "2018-06-16",
            "2019-06-06",
            "2020-05-25",
            "2021-05-14",
            "2022-05-03",
            "2023-04-23",
            "2024-04-11",
            "2025-04-01",
        )
        self.assertIslamicNoEstimatedNonObservedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "Hari Raya Qurban"
        self.assertHolidayName(
            name,
            "2012-10-26",
            "2013-10-15",
            "2014-10-05",
            "2015-09-24",
            "2016-09-12",
            "2017-09-01",
            "2018-08-22",
            "2019-08-11",
            "2020-07-31",
            "2021-07-20",
            "2022-07-10",
            "2023-06-29",
            "2024-06-17",
            "2025-06-07",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_isra_and_miraj(self):
        name = "Israk dan Mikraj"
        self.assertNoHolidayName(name)
        dt_1 = (
            "2012-06-17",
            "2013-06-06",
            "2014-05-27",
            "2015-05-16",
            "2016-05-05",
            "2017-04-24",
            "2018-04-14",
            "2019-04-03",
            "2020-03-22",
            "2021-03-11",
            "2022-03-01",
            "2023-02-18",
            "2024-02-08",
        )
        dt_2 = (
            "2020-03-22",
            "2021-03-11",
            "2022-03-01",
            "2023-02-18",
            "2024-02-08",
            "2025-01-27",
        )
        for subdiv, holidays in self.subdiv_islamic_no_estimated_holidays.items():
            match subdiv:
                case "02" | "05" | "09":
                    self.assertHolidayName(name, holidays, dt_1, dt_2)
                    self.assertHolidayName(name, holidays, self.full_range)
                case "11":
                    self.assertHolidayName(name, holidays, dt_2)
                    self.assertHolidayName(name, holidays, range(2020, self.end_year))
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 2020))
                case _:
                    self.assertNoHolidayName(name, holidays)

    def test_beginning_of_ramadan(self):
        name = "Awal Ramadan"
        self.assertNoHolidayName(name)
        dt_1 = (
            "2012-07-20",
            "2013-07-09",
            "2014-06-29",
            "2015-06-18",
            "2016-06-07",
            "2017-05-27",
            "2018-05-17",
            "2019-05-06",
            "2020-04-24",
            "2021-04-13",
            "2022-04-03",
            "2023-03-23",
            "2024-03-12",
        )
        dt_2 = ("2025-03-02",)
        for subdiv, holidays in self.subdiv_islamic_no_estimated_holidays.items():
            match subdiv:
                case "01" | "02":
                    self.assertHolidayName(name, holidays, dt_1, dt_2)
                    self.assertHolidayName(name, holidays, self.full_range)
                case "04":
                    self.assertHolidayName(name, holidays, dt_1)
                    self.assertHolidayName(name, holidays, range(self.start_year, 2025))
                    self.assertNoHolidayName(name, holidays, range(2025, self.end_year))
                case _:
                    self.assertNoHolidayName(name, holidays)

    def test_eid_al_fitr_third_day(self):
        name = "Hari Raya Puasa (Hari Ketiga)"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_islamic_no_estimated_holidays.items():
            if subdiv == "04":
                self.assertHolidayName(name, holidays, "2025-04-02")
                self.assertHolidayName(name, holidays, range(2025, self.end_year))
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2025))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_nuzul_al_quran_day(self):
        name = "Hari Nuzul Al-Quran"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_islamic_no_estimated_holidays.items():
            if subdiv in {"03", "06", "07", "08", "09", "10", "11", "14", "15", "16"}:
                self.assertHolidayName(
                    name,
                    holidays,
                    "2012-08-05",
                    "2013-07-25",
                    "2014-07-15",
                    "2015-07-04",
                    "2016-06-22",
                    "2017-06-12",
                    "2018-06-02",
                    "2019-05-22",
                    "2020-05-10",
                    "2021-04-29",
                    "2022-04-19",
                    "2023-04-08",
                    "2024-03-28",
                    "2025-03-18",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_arafat_day(self):
        name = "Hari Arafah"
        self.assertNoHolidayName(name)
        dt_1 = (
            "2012-10-25",
            "2013-10-14",
            "2014-10-04",
            "2015-09-23",
            "2016-09-11",
            "2017-08-31",
            "2018-08-21",
            "2019-08-10",
            "2020-07-30",
            "2021-07-19",
            "2022-07-09",
        )
        dt_2 = (
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )

        for subdiv, holidays in self.subdiv_islamic_no_estimated_holidays.items():
            match subdiv:
                case "03":
                    self.assertHolidayName(name, holidays, dt_2)
                    self.assertHolidayName(name, holidays, range(2023, self.end_year))
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 2023))
                case "11":
                    self.assertHolidayName(name, holidays, dt_1, dt_2)
                    self.assertHolidayName(name, holidays, self.full_range)
                case _:
                    self.assertNoHolidayName(name, holidays)

    def test_eid_al_adha_second_day(self):
        name = "Hari Raya Qurban (Hari Kedua)"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_islamic_no_estimated_holidays.items():
            if subdiv in {"02", "03", "09", "11"}:
                self.assertHolidayName(
                    name,
                    holidays,
                    "2012-10-27",
                    "2013-10-16",
                    "2014-10-06",
                    "2015-09-25",
                    "2016-09-13",
                    "2017-09-02",
                    "2018-08-23",
                    "2019-08-12",
                    "2020-08-01",
                    "2021-07-21",
                    "2022-07-11",
                    "2023-06-30",
                    "2024-06-18",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_observed(self):
        dt = (
            "2012-02-06",
            "2012-08-21",
            "2012-09-17",
            "2013-02-12",
            "2014-09-01",
            "2014-10-06",
            "2015-05-04",
            "2016-05-02",
            "2016-12-26",
            "2017-01-30",
            "2017-06-27",
            "2018-09-10",
            "2018-09-17",
            "2019-05-20",
            "2019-08-12",
            "2020-01-27",
            "2020-05-26",
            "2022-05-04",
            "2022-05-16",
            "2022-07-11",
            "2022-12-26",
            "2023-01-24",
            "2023-04-24",
            "2024-02-12",
        )
        self.assertHoliday(dt)
        self.assertNoNonObservedHoliday(dt)

    def test_birthday_of_sultan_of_johor(self):
        name = "Hari Keputeraan Sultan Johor"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "01":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-23" for year in range(2015, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2015))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_sultan_of_johor_hol(self):
        name = "Hari Hol Almarhum Sultan Iskandar"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_islamic_no_estimated_holidays.items():
            if subdiv == "01":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2011-01-12",
                    "2012-12-20",
                    "2013-12-10",
                    "2014-11-29",
                    "2015-11-19",
                    "2016-11-07",
                    "2017-10-27",
                    "2018-10-15",
                    "2019-10-05",
                    "2020-09-24",
                    "2021-09-13",
                    "2022-09-03",
                    "2023-08-23",
                    "2024-08-11",
                    "2025-07-31",
                )
                self.assertHolidayName(name, holidays, range(2011, self.end_year))
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2011))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_birthday_of_sultan_of_kedah(self):
        name = "Hari Keputeraan Sultan Kedah"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "02":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2018-06-17",
                    "2019-06-16",
                    "2020-06-21",
                    "2021-06-20",
                    "2022-06-19",
                    "2023-06-18",
                    "2024-06-30",
                    "2025-06-15",
                )
                self.assertHolidayName(name, holidays, range(2018, self.end_year))
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2018))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_birthday_of_sultan_of_kelantan(self):
        name = "Hari Keputeraan Sultan Kelantan"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "03":
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-03-30" for year in range(2010, 2012)),
                    (f"{year}-03-31" for year in range(2010, 2012)),
                )
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-11-11" for year in range(2012, 2023)),
                    (f"{year}-11-12" for year in range(2012, 2023)),
                )
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-09-29" for year in range(2023, self.end_year)),
                    (f"{year}-09-30" for year in range(2023, self.end_year)),
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2010))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_malacca_declaration_of_independence_day(self):
        name = "Hari Pengisytiharan Tarikh Kemerdekaan"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "04":
                self.assertHolidayName(
                    name, holidays, (f"{year}-02-20" for year in range(2024, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2024))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_declaration_malacca_as_historical_city(self):
        name = "Hari Perisytiharan Melaka Sebagai Bandaraya Bersejarah"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "04":
                self.assertHolidayName(
                    name, holidays, (f"{year}-04-15" for year in range(1989, 2024))
                )
                self.assertNoHolidayName(
                    name, holidays, range(self.start_year, 1989), range(2024, self.end_year)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_birthday_of_governor_of_malacca(self):
        name = "Hari Jadi Yang di-Pertua Negeri Melaka"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "04":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-24" for year in range(2020, self.end_year))
                )
                self.assertHolidayName(
                    name,
                    holidays,
                    "2012-10-12",
                    "2013-10-11",
                    "2014-10-10",
                    "2015-10-09",
                    "2016-10-14",
                    "2017-10-13",
                    "2018-10-12",
                    "2019-10-11",
                )
                self.assertHolidayName(name, holidays, range(self.start_year, 2025))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_birthday_of_sultan_of_negeri_sembilan(self):
        name = "Hari Keputeraan Yang di-Pertuan Besar Negeri Sembilan"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "05":
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-14" for year in range(2009, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2009))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_sultan_of_pahang_hol(self):
        name = "Hari Hol Sultan Pahang"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "06":
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-05-22" for year in range(2020, self.end_year)),
                    (f"{year}-05-07" for year in range(1975, 2020)),
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1975))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_birthday_of_sultan_of_pahang(self):
        name = "Hari Keputeraan Sultan Pahang"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "06":
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-07-30" for year in range(2019, self.end_year)),
                    (f"{year}-10-24" for year in range(1975, 2019)),
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1975))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_george_town_heritage_day(self):
        name = "Hari Ulang Tahun Perisytiharan Tapak Warisan Dunia"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "07":
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-07" for year in range(2009, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2009))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_birthday_of_governor_of_penang(self):
        name = "Hari Jadi Yang di-Pertua Negeri Pulau Pinang"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "07":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2012-07-14",
                    "2013-07-13",
                    "2014-07-12",
                    "2015-07-11",
                    "2016-07-09",
                    "2017-07-08",
                    "2018-07-14",
                    "2019-07-13",
                    "2020-07-11",
                    "2021-07-10",
                    "2022-07-09",
                    "2023-07-08",
                    "2024-07-13",
                    "2025-07-12",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_birthday_of_sultan_of_perak(self):
        name = "Hari Keputeraan Sultan Perak"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "08":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-27" for year in range(self.start_year, 2018))
                )
                self.assertHolidayName(
                    name,
                    holidays,
                    "2018-11-02",
                    "2019-11-01",
                    "2020-11-06",
                    "2021-11-05",
                    "2022-11-04",
                    "2023-11-03",
                    "2024-11-01",
                    "2025-11-07",
                )
                self.assertHolidayName(name, holidays, range(2018, self.end_year))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_birthday_of_raja_of_perlis(self):
        name = "Hari Ulang Tahun Keputeraan Raja Perlis"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "09":
                self.assertHolidayName(
                    name,
                    holidays,
                    (
                        f"{year}-05-17"
                        for year in (*range(2000, 2018), *range(2022, self.end_year))
                    ),
                    (f"{year}-07-17" for year in range(2018, 2022)),
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2000))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_birthday_of_sultan_of_selangor(self):
        name = "Hari Keputeraan Sultan Selangor"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "10":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-11" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_installation_of_sultan_of_terengganu(self):
        name = "Hari Ulang Tahun Pertabalan Sultan Terengganu"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "11":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-04" for year in range(2000, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2000))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_birthday_of_sultan_of_terengganu(self):
        name = "Hari Keputeraan Sultan Terengganu"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "11":
                self.assertHolidayName(
                    name, holidays, (f"{year}-04-26" for year in range(2000, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2000))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_birthday_of_governor_of_sabah(self):
        name = "Hari Jadi Yang di-Pertua Negeri Sabah"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "12":
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-09-16" for year in (*range(2001, 2005), *range(2006, 2010))),
                    (f"{year}-03-30" for year in range(2025, self.end_year)),
                )
                self.assertHolidayName(
                    name,
                    holidays,
                    "2005-02-11",
                    "2010-10-02",
                    "2011-10-01",
                    "2012-10-06",
                    "2013-10-05",
                    "2014-10-04",
                    "2015-10-03",
                    "2016-10-01",
                    "2017-10-07",
                    "2018-10-06",
                    "2019-10-05",
                    "2020-10-03",
                    "2021-10-02",
                    "2022-10-01",
                    "2023-10-07",
                    "2024-10-05",
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2001))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_christmas_eve(self):
        name = "Krismas (Eve)"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "12":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-24" for year in range(2019, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2019))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_dayak_festival_day(self):
        name = "Perayaan Hari Gawai Dayak"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "13":
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-06-01" for year in range(1965, self.end_year)),
                    (f"{year}-06-02" for year in range(1965, self.end_year)),
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1965))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_birthday_of_governor_of_sarawak(self):
        name = "Hari Jadi Yang di-Pertua Negeri Sarawak"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "13":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2012-10-13",
                    "2013-10-12",
                    "2014-10-11",
                    "2015-10-10",
                    "2016-10-08",
                    "2017-10-14",
                    "2018-10-13",
                    "2019-10-12",
                    "2020-10-10",
                    "2021-10-09",
                    "2022-10-08",
                    "2023-10-14",
                    "2024-10-12",
                    "2025-10-11",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_sarawak_independence_day(self):
        name = "Hari Kemerdekaan Sarawak"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "13":
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-22" for year in range(2017, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2017))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_weekend(self):
        for dt in (
            "1994-12-31",  # SAT.
            "1995-01-01",  # SUN.
            "1995-01-07",  # SAT.
            "1995-01-08",  # SUN.
            "2013-12-28",  # SAT.
            "2013-12-29",  # SUN.
            "2014-01-04",  # SAT.
            "2014-01-05",  # SUN.
            "2024-12-28",  # SAT.
            "2024-12-29",  # SUN.
            "2025-01-04",  # SAT.
            "2025-01-05",  # SUN.
        ):
            self.assertTrue(self.holidays.is_weekend(dt))

        for dt in (
            "1994-12-30",  # FRI.
            "1995-01-06",  # FRI.
            "2013-12-27",  # FRI.
            "2014-01-03",  # FRI.
            "2024-12-27",  # FRI.
            "2025-01-03",  # FRI.
        ):
            self.assertFalse(self.holidays.is_weekend(dt))

        for subdiv in ("02", "03", "11"):
            subdiv_holidays = self.subdiv_holidays[subdiv]

            for dt in (
                "1994-12-30",  # FRI.
                "1994-12-31",  # SAT.
                "1995-01-06",  # FRI.
                "1995-01-07",  # SAT.
                "2013-12-27",  # FRI.
                "2013-12-28",  # SAT.
                "2014-01-03",  # FRI.
                "2014-01-04",  # SAT.
                "2024-12-27",  # FRI.
                "2024-12-28",  # SAT.
                "2025-01-03",  # FRI.
                "2025-01-04",  # SAT.
            ):
                self.assertTrue(subdiv_holidays.is_weekend(dt))

            for dt in (
                "1994-12-25",  # SUN.
                "1995-01-01",  # SUN.
                "2013-12-29",  # SUN.
                "2014-01-05",  # SUN.
                "2024-12-29",  # SUN.
                "2025-01-05",  # SUN.
            ):
                self.assertFalse(subdiv_holidays.is_weekend(dt))

        subdiv_01_holidays = self.subdiv_holidays["01"]

        for dt in (
            "1994-12-30",  # FRI.
            "1994-12-31",  # SAT.
            "1995-01-01",  # SUN.
            "1995-01-07",  # SAT.
            "1995-01-08",  # SUN.
            "2013-12-28",  # SAT.
            "2013-12-29",  # SUN.
            "2014-01-03",  # FRI.
            "2014-01-04",  # SAT.
            "2024-12-27",  # FRI.
            "2024-12-27",  # SAT.
            "2025-01-04",  # SAT.
            "2025-01-05",  # SUN.
        ):
            self.assertTrue(subdiv_01_holidays.is_weekend(dt))

        for dt in (
            "1994-12-25",  # SUN.
            "1995-01-06",  # FRI.
            "2013-12-27",  # FRI.
            "2014-01-05",  # SUN.
            "2024-12-29",  # SUN.
            "2025-01-03",  # FRI.
        ):
            self.assertFalse(subdiv_01_holidays.is_weekend(dt))

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-02-10", "Tahun Baharu Cina"),
            ("2024-02-11", "Tahun Baharu Cina (Hari Kedua)"),
            ("2024-02-12", "Cuti Tahun Baharu Cina (Hari Kedua)"),
            ("2024-04-10", "Hari Raya Puasa"),
            ("2024-04-11", "Hari Raya Puasa (Hari Kedua)"),
            ("2024-05-01", "Hari Pekerja"),
            ("2024-05-22", "Hari Wesak"),
            ("2024-06-03", "Hari Keputeraan Rasmi Seri Paduka Baginda Yang di-Pertuan Agong"),
            ("2024-06-17", "Hari Raya Qurban"),
            ("2024-07-07", "Awal Muharam"),
            ("2024-08-31", "Hari Kebangsaan"),
            ("2024-09-16", "Hari Keputeraan Nabi Muhammad S.A.W.; Hari Malaysia"),
            ("2024-12-25", "Hari Krismas"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Tahun Baharu"),
            ("2023-01-02", "Cuti Tahun Baharu"),
            ("2023-01-14", "Hari Keputeraan Yang di-Pertuan Besar Negeri Sembilan"),
            ("2023-01-22", "Tahun Baharu Cina"),
            ("2023-01-23", "Tahun Baharu Cina (Hari Kedua)"),
            ("2023-01-24", "Cuti Tahun Baharu Cina"),
            ("2023-02-01", "Hari Wilayah Persekutuan"),
            ("2023-02-05", "Hari Thaipusam"),
            ("2023-02-06", "Cuti Hari Thaipusam"),
            ("2023-02-18", "Israk dan Mikraj"),
            ("2023-02-19", "Cuti Israk dan Mikraj"),
            ("2023-03-04", "Hari Ulang Tahun Pertabalan Sultan Terengganu"),
            ("2023-03-05", "Cuti Hari Ulang Tahun Pertabalan Sultan Terengganu"),
            ("2023-03-23", "Awal Ramadan; Hari Keputeraan Sultan Johor"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-08", "Hari Nuzul Al-Quran"),
            ("2023-04-09", "Cuti Hari Nuzul Al-Quran"),
            ("2023-04-15", "Hari Perisytiharan Melaka Sebagai Bandaraya Bersejarah"),
            ("2023-04-21", "Hari Raya Puasa (pergantian hari)"),
            ("2023-04-22", "Hari Raya Puasa"),
            ("2023-04-23", "Hari Raya Puasa (Hari Kedua)"),
            ("2023-04-24", "Cuti Hari Raya Puasa; Cuti Hari Raya Puasa (Hari Kedua)"),
            ("2023-04-26", "Hari Keputeraan Sultan Terengganu"),
            ("2023-05-01", "Hari Pekerja"),
            ("2023-05-04", "Hari Wesak"),
            ("2023-05-17", "Hari Ulang Tahun Keputeraan Raja Perlis"),
            ("2023-05-22", "Hari Hol Sultan Pahang"),
            ("2023-05-30", "Pesta Kaamatan"),
            ("2023-05-31", "Pesta Kaamatan"),
            ("2023-06-01", "Perayaan Hari Gawai Dayak"),
            ("2023-06-02", "Perayaan Hari Gawai Dayak"),
            ("2023-06-05", "Hari Keputeraan Rasmi Seri Paduka Baginda Yang di-Pertuan Agong"),
            ("2023-06-18", "Hari Keputeraan Sultan Kedah"),
            ("2023-06-28", "Hari Arafah"),
            ("2023-06-29", "Hari Raya Qurban"),
            ("2023-06-30", "Hari Raya Qurban (Hari Kedua)"),
            ("2023-07-02", "Cuti Hari Raya Qurban (Hari Kedua)"),
            ("2023-07-07", "Hari Ulang Tahun Perisytiharan Tapak Warisan Dunia"),
            ("2023-07-08", "Hari Jadi Yang di-Pertua Negeri Pulau Pinang"),
            ("2023-07-19", "Awal Muharam"),
            ("2023-07-22", "Hari Kemerdekaan Sarawak"),
            ("2023-07-30", "Hari Keputeraan Sultan Pahang"),
            ("2023-07-31", "Cuti Hari Keputeraan Sultan Pahang"),
            ("2023-08-23", "Hari Hol Almarhum Sultan Iskandar"),
            ("2023-08-24", "Hari Jadi Yang di-Pertua Negeri Melaka"),
            ("2023-08-31", "Hari Kebangsaan"),
            ("2023-09-16", "Hari Malaysia"),
            ("2023-09-17", "Cuti Hari Malaysia"),
            ("2023-09-28", "Hari Keputeraan Nabi Muhammad S.A.W."),
            ("2023-09-29", "Hari Keputeraan Sultan Kelantan"),
            ("2023-09-30", "Hari Keputeraan Sultan Kelantan"),
            ("2023-10-07", "Hari Jadi Yang di-Pertua Negeri Sabah"),
            ("2023-10-14", "Hari Jadi Yang di-Pertua Negeri Sarawak"),
            ("2023-11-03", "Hari Keputeraan Sultan Perak"),
            ("2023-11-12", "Hari Deepavali"),
            ("2023-11-13", "Cuti Hari Deepavali"),
            ("2023-12-11", "Hari Keputeraan Sultan Selangor"),
            ("2023-12-24", "Krismas (Eve)"),
            ("2023-12-25", "Hari Krismas"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-01-14", "Birthday of the Sultan of Negeri Sembilan"),
            ("2023-01-22", "Chinese New Year"),
            ("2023-01-23", "Chinese New Year (Second Day)"),
            ("2023-01-24", "Chinese New Year (observed)"),
            ("2023-02-01", "Federal Territory Day"),
            ("2023-02-05", "Thaipusam"),
            ("2023-02-06", "Thaipusam (observed)"),
            ("2023-02-18", "Isra' and Mi'raj"),
            ("2023-02-19", "Isra' and Mi'raj (observed)"),
            ("2023-03-04", "Anniversary of the Installation of the Sultan of Terengganu"),
            (
                "2023-03-05",
                "Anniversary of the Installation of the Sultan of Terengganu (observed)",
            ),
            ("2023-03-23", "Beginning of Ramadan; Birthday of the Sultan of Johor"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-08", "Nuzul Al-Quran Day"),
            ("2023-04-09", "Nuzul Al-Quran Day (observed)"),
            ("2023-04-15", "Declaration of Malacca as a Historical City"),
            ("2023-04-21", "Eid al-Fitr (additional holiday)"),
            ("2023-04-22", "Eid al-Fitr"),
            ("2023-04-23", "Eid al-Fitr (Second Day)"),
            ("2023-04-24", "Eid al-Fitr (Second Day) (observed); Eid al-Fitr (observed)"),
            ("2023-04-26", "Birthday of the Sultan of Terengganu"),
            ("2023-05-01", "Labor Day"),
            ("2023-05-04", "Vesak Day"),
            ("2023-05-17", "Birthday of the Raja of Perlis"),
            ("2023-05-22", "The Sultan of Pahang Hol"),
            ("2023-05-30", "Pesta Kaamatan"),
            ("2023-05-31", "Pesta Kaamatan"),
            ("2023-06-01", "Dayak Festival Day"),
            ("2023-06-02", "Dayak Festival Day"),
            ("2023-06-05", "Birthday of HM Yang di-Pertuan Agong"),
            ("2023-06-18", "Birthday of The Sultan of Kedah"),
            ("2023-06-28", "Arafat Day"),
            ("2023-06-29", "Eid al-Adha"),
            ("2023-06-30", "Eid al-Adha (Second Day)"),
            ("2023-07-02", "Eid al-Adha (Second Day) (observed)"),
            ("2023-07-07", "George Town Heritage Day"),
            ("2023-07-08", "Birthday of the Governor of Penang"),
            ("2023-07-19", "Islamic New Year"),
            ("2023-07-22", "Sarawak Independence Day"),
            ("2023-07-30", "Birthday of the Sultan of Pahang"),
            ("2023-07-31", "Birthday of the Sultan of Pahang (observed)"),
            ("2023-08-23", "The Sultan of Johor Hol"),
            ("2023-08-24", "Birthday of the Governor of Malacca"),
            ("2023-08-31", "National Day"),
            ("2023-09-16", "Malaysia Day"),
            ("2023-09-17", "Malaysia Day (observed)"),
            ("2023-09-28", "Prophet Muhammad's Birthday"),
            ("2023-09-29", "Birthday of the Sultan of Kelantan"),
            ("2023-09-30", "Birthday of the Sultan of Kelantan"),
            ("2023-10-07", "Birthday of the Governor of Sabah"),
            ("2023-10-14", "Birthday of the Governor of Sarawak"),
            ("2023-11-03", "Birthday of the Sultan of Perak"),
            ("2023-11-12", "Deepavali"),
            ("2023-11-13", "Deepavali (observed)"),
            ("2023-12-11", "Birthday of The Sultan of Selangor"),
            ("2023-12-24", "Christmas Eve"),
            ("2023-12-25", "Christmas Day"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2023-01-01", "วันขึ้นปีใหม่"),
            ("2023-01-02", "ชดเชยวันขึ้นปีใหม่"),
            ("2023-01-14", "วันคล้ายวันประสูติสุลต่านแห่งรัฐเนเกรีเซมบิลัน"),
            ("2023-01-22", "วันตรุษจีน"),
            ("2023-01-23", "วันตรุษจีนวันที่สอง"),
            ("2023-01-24", "ชดเชยวันตรุษจีน"),
            ("2023-02-01", "วันเขตสหพันธรัฐ"),
            ("2023-02-05", "วันไทปูซัม"),
            ("2023-02-06", "ชดเชยวันไทปูซัม"),
            ("2023-02-18", "วันเมี๊ยะราจ"),
            ("2023-02-19", "ชดเชยวันเมี๊ยะราจ"),
            ("2023-03-04", "วันครบรอบพระราชพิธีสถาปนาสุลต่านแห่งรัฐตรังกานู"),
            ("2023-03-05", "ชดเชยวันครบรอบพระราชพิธีสถาปนาสุลต่านแห่งรัฐตรังกานู"),
            ("2023-03-23", "วันคล้ายวันพระราชสมภพสุลต่านแห่งรัฐยะโฮร์; วันแรกการถือศีลอด"),
            ("2023-04-07", "วันศุกร์ประเสริฐ"),
            ("2023-04-08", "วันนูซุลอัลกุรอาน"),
            ("2023-04-09", "ชดเชยวันนูซุลอัลกุรอาน"),
            ("2023-04-15", "วันรำลึกการประกาศมะละกาเป็นเมืองประวัติศาสตร์"),
            ("2023-04-21", "วันอีฎิ้ลฟิตริ (เพิ่มเติม)"),
            ("2023-04-22", "วันอีฎิ้ลฟิตริ"),
            ("2023-04-23", "วันอีฎิ้ลฟิตริวันที่สอง"),
            ("2023-04-24", "ชดเชยวันอีฎิ้ลฟิตริ; ชดเชยวันอีฎิ้ลฟิตริวันที่สอง"),
            ("2023-04-26", "วันคล้ายวันประสูติสุลต่านแห่งรัฐตรังกานู"),
            ("2023-05-01", "วันแรงงาน"),
            ("2023-05-04", "วันวิสาขบูชา"),
            ("2023-05-17", "วันคล้ายวันประสูติสุลต่านแห่งรัฐปะลิส"),
            ("2023-05-22", "วันคล้ายวันสิ้นพระชนม์สุลต่านแห่งรัฐปะหังองค์ก่อน"),
            ("2023-05-30", "เทศกาลกามะตัน"),
            ("2023-05-31", "เทศกาลกามะตัน"),
            ("2023-06-01", "วันเทศกาลกาไวดายัค"),
            ("2023-06-02", "วันเทศกาลกาไวดายัค"),
            ("2023-06-05", "วันคล้ายวันพระราชสมภพสมเด็จพระราชาธิบดีแห่งมาเลเซีย"),
            ("2023-06-18", "วันคล้ายวันประสูติสุลต่านแห่งรัฐเกดะห์"),
            ("2023-06-28", "วันอารอฟะห์"),
            ("2023-06-29", "วันอีดิ้ลอัฎฮา"),
            ("2023-06-30", "วันอีดิ้ลอัฎฮาวันที่สอง"),
            ("2023-07-02", "ชดเชยวันอีดิ้ลอัฎฮาวันที่สอง"),
            ("2023-07-07", "วันครบรอบการประกาศจอร์จทาวน์เป็นมรดกโลก"),
            ("2023-07-08", "วันคล้ายวันเกิดผู้ว่าการรัฐปีนัง"),
            ("2023-07-19", "วันขึ้นปีใหม่อิสลาม"),
            ("2023-07-22", "วันประกาศเอกราชรัฐซาราวัก"),
            ("2023-07-30", "วันคล้ายวันประสูติสุลต่านแห่งรัฐปะหัง"),
            ("2023-07-31", "ชดเชยวันคล้ายวันประสูติสุลต่านแห่งรัฐปะหัง"),
            ("2023-08-23", "วันคล้ายวันสวรรคตสุลต่านแห่งรัฐยะโฮร์องค์ก่อน"),
            ("2023-08-24", "วันคล้ายวันเกิดผู้ว่าการรัฐมะละกา"),
            ("2023-08-31", "วันชาติมาเลเซีย"),
            ("2023-09-16", "วันเฉลิมฉลองการจัดตั้งสหพันธรัฐมาเลเซีย"),
            ("2023-09-17", "ชดเชยวันเฉลิมฉลองการจัดตั้งสหพันธรัฐมาเลเซีย"),
            ("2023-09-28", "วันเมาลิดนบี"),
            ("2023-09-29", "วันคล้ายวันประสูติสุลต่านแห่งรัฐกลันตัน"),
            ("2023-09-30", "วันคล้ายวันประสูติสุลต่านแห่งรัฐกลันตัน"),
            ("2023-10-07", "วันคล้ายวันเกิดผู้ว่าการรัฐซาบาห์"),
            ("2023-10-14", "วันคล้ายวันเกิดผู้ว่าการรัฐซาราวัก"),
            ("2023-11-03", "วันคล้ายวันประสูติสุลต่านแห่งรัฐเประก์"),
            ("2023-11-12", "วันดีปาวลี"),
            ("2023-11-13", "ชดเชยวันดีปาวลี"),
            ("2023-12-11", "วันคล้ายวันประสูติสุลต่านแห่งรัฐสลังงอร์"),
            ("2023-12-24", "วันคริสต์มาสอีฟ"),
            ("2023-12-25", "วันคริสต์มาส"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.sudan import Sudan
from tests.common import CommonCountryTests


class TestSudan(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Sudan)

    def test_independence_day(self):
        self.assertHolidayName(
            "عيد الإستقلال", (f"{year}-01-01" for year in range(self.start_year, self.end_year))
        )

    def test_coptic_christmas(self):
        name = "عيد الميلاد المجيد"
        self.assertHolidayName(
            name,
            (
                f"{year}-01-07"
                for year in (*range(self.start_year, 2011), *range(2019, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(2011, 2019))

    def test_christmas_day(self):
        name = "عيد الميلاد"
        self.assertHolidayName(
            name,
            (
                f"{year}-12-25"
                for year in (*range(self.start_year, 2011), *range(2019, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(2011, 2019))

    def test_coptic_easter(self):
        name = "عيد الفصح القبطي"
        self.assertHolidayName(
            name,
            "2019-04-28",
            "2020-04-19",
            "2021-05-02",
            "2022-04-24",
            "2023-04-16",
            "2024-05-05",
            "2025-04-20",
        )
        self.assertHolidayName(name, range(self.start_year, 2011), range(2019, self.end_year))
        self.assertNoHolidayName(name, range(2011, 2019))

    def test_islamic_new_year(self):
        name = "رأس السنة الهجرية"
        self.assertHolidayName(
            name,
            "2020-08-20",
            "2021-08-11",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(self.start_year, self.end_year))

    def test_prophets_birthday(self):
        name = "المولد النبوي الشريف"
        self.assertHolidayName(
            name,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-28",
            "2024-09-15",
            "2025-09-04",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(self.start_year, self.end_year))

    def test_eid_al_fitr(self):
        name = "عيد الفطر المبارك"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-01",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(self.start_year, self.end_year))
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 3, set(range(self.start_year, 2020)) - {2000}
        )
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 4, set(range(2020, self.end_year)) - {2033}
        )

    def test_eid_al_adha(self):
        name = "عيد الأضحى المبارك"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-10",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(self.start_year, self.end_year))
        self.assertIslamicNoEstimatedHolidayNameCount(
            name,
            5,
            set(range(self.start_year, self.end_year)) - {2006, 2007, 2039},
        )

    def test_weekend(self):
        for dt in (
            "2008-01-18",  # FRI.
            "2008-01-25",  # FRI.
            "2008-01-26",  # SAT.
            "2008-02-01",  # FRI.
            "2008-02-02",  # SAT.
        ):
            self.assertTrue(self.holidays.is_weekend(dt))

        for dt in (
            "2008-01-17",  # THU.
            "2008-01-19",  # SAT.
            "2008-01-20",  # SUN.
            "2008-01-24",  # THU.
            "2008-01-27",  # SUN.
        ):
            self.assertFalse(self.holidays.is_weekend(dt))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "عيد الإستقلال"),
            ("2022-01-07", "عيد الميلاد المجيد"),
            ("2022-04-24", "عيد الفصح القبطي"),
            ("2022-05-01", "عيد الفطر المبارك"),
            ("2022-05-02", "عيد الفطر المبارك"),
            ("2022-05-03", "عيد الفطر المبارك"),
            ("2022-05-04", "عيد الفطر المبارك"),
            ("2022-07-09", "عيد الأضحى المبارك"),
            ("2022-07-10", "عيد الأضحى المبارك"),
            ("2022-07-11", "عيد الأضحى المبارك"),
            ("2022-07-12", "عيد الأضحى المبارك"),
            ("2022-07-13", "عيد الأضحى المبارك"),
            ("2022-07-30", "رأس السنة الهجرية"),
            ("2022-10-08", "المولد النبوي الشريف"),
            ("2022-12-25", "عيد الميلاد"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "Independence Day"),
            ("2022-01-07", "Coptic Christmas"),
            ("2022-04-24", "Coptic Easter"),
            ("2022-05-01", "Eid al-Fitr"),
            ("2022-05-02", "Eid al-Fitr"),
            ("2022-05-03", "Eid al-Fitr"),
            ("2022-05-04", "Eid al-Fitr"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-07-10", "Eid al-Adha"),
            ("2022-07-11", "Eid al-Adha"),
            ("2022-07-12", "Eid al-Adha"),
            ("2022-07-13", "Eid al-Adha"),
            ("2022-07-30", "Islamic New Year"),
            ("2022-10-08", "Prophet's Birthday"),
            ("2022-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.financial.national_stock_exchange_of_india import NationalStockExchangeOfIndia
from tests.common import CommonFinancialTests


class TestNationalStockExchangeOfIndia(CommonFinancialTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(NationalStockExchangeOfIndia)

    def test_special_holidays(self):
        self.assertHoliday(
            "2010-01-01",
        )
        self.assertNoHolidayName(
            "New Year", range(self.start_year, 2010), range(2011, self.end_year)
        )

    def test_republic_day(self):
        self.assertNonObservedHolidayName(
            "Republic Day", (f"{year}-01-26" for year in self.full_range)
        )
        self.assertNoHoliday(
            "2013-01-26",
            "2014-01-26",
            "2019-01-26",
            "2020-01-26",
            "2025-01-26",
        )

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_dr_baba_saheb_ambedkar_jayanti(self):
        self.assertNonObservedHolidayName(
            "Dr. Baba Saheb Ambedkar Jayanti", (f"{year}-04-14" for year in self.full_range)
        )
        self.assertNoHoliday(
            "2012-04-14",
            "2013-04-14",
            "2018-04-14",
            "2019-04-14",
            "2024-04-14",
        )

    def test_maharashtra_day(self):
        name_2001 = "Maharashtra Day"
        name_2010 = "May Day"
        self.assertNonObservedHolidayName(
            name_2001,
            (
                f"{year}-05-01"
                for year in (*range(self.start_year, 2010), *range(2015, self.end_year))
            ),
        )
        self.assertNonObservedHolidayName(
            name_2010, (f"{year}-05-01" for year in range(2010, 2015))
        )
        self.assertNoNonObservedHolidayName(name_2001, range(2010, 2015))
        self.assertNoNonObservedHolidayName(
            name_2010, range(self.start_year, 2010), range(2015, self.end_year)
        )
        self.assertNoHoliday(
            "2004-05-01",
            "2010-05-01",
            "2011-05-01",
            "2016-05-01",
            "2021-05-01",
            "2022-05-01",
        )

    def test_independence_day(self):
        self.assertNonObservedHolidayName(
            "Independence Day", (f"{year}-08-15" for year in self.full_range)
        )
        self.assertNoHoliday(
            "2015-08-15",
            "2020-08-15",
            "2021-08-15",
        )

    def test_mahatma_gandhi_jayanti(self):
        self.assertNonObservedHolidayName(
            "Mahatma Gandhi Jayanti", (f"{year}-10-02" for year in self.full_range)
        )
        self.assertNoHoliday(
            "2010-10-02",
            "2011-10-02",
            "2016-10-02",
            "2021-10-02",
            "2022-10-02",
        )

    def test_christmas_day(self):
        self.assertNonObservedHolidayName(
            "Christmas Day", (f"{year}-12-25" for year in self.full_range)
        )
        self.assertNoHoliday(
            "2010-12-25",
            "2011-12-25",
            "2016-12-25",
            "2021-12-25",
            "2022-12-25",
        )

    def test_maha_shivaratri(self):
        name = "Maha Shivaratri"
        self.assertNonObservedHolidayName(
            name,
            "2020-02-21",
            "2021-03-11",
            "2022-03-01",
            "2023-02-18",
            "2024-03-08",
            "2025-02-26",
        )
        self.assertNonObservedHolidayName(name, range(2007, 2036))
        self.assertNoNonObservedHolidayName(name, range(self.start_year, 2007))
        self.assertNoHoliday(
            "2013-03-10",
            "2023-02-18",
        )

    def test_holi(self):
        name = "Holi"
        self.assertNonObservedHolidayName(
            name,
            "2020-03-10",
            "2021-03-29",
            "2022-03-18",
            "2023-03-07",
            "2024-03-25",
            "2025-03-14",
        )
        self.assertNonObservedHolidayName(name, range(self.start_year, 2036))
        self.assertNoHoliday(
            "2008-03-22",
            "2011-03-20",
        )

    def test_ram_navami(self):
        name = "Ram Navami"
        self.assertNonObservedHolidayName(
            name,
            "2020-04-02",
            "2021-04-21",
            "2022-04-10",
            "2023-03-30",
            "2024-04-17",
            "2025-04-06",
        )
        self.assertNonObservedHolidayName(name, range(2006, 2036))
        self.assertNoNonObservedHolidayName(name, range(self.start_year, 2006))
        self.assertNoHoliday(
            "2012-04-01",
            "2015-03-28",
            "2018-03-25",
            "2019-04-13",
            "2022-04-10",
            "2025-04-06",
        )

    def test_mahavir_jayanti(self):
        name = "Mahavir Jayanti"
        self.assertNonObservedHolidayName(
            name,
            "2020-04-06",
            "2021-04-25",
            "2022-04-14",
            "2023-04-04",
            "2024-04-21",
            "2025-04-10",
        )
        self.assertNonObservedHolidayName(name, range(2006, 2036))
        self.assertNoNonObservedHolidayName(name, range(self.start_year, 2006))
        self.assertNoHoliday(
            "2010-03-28",
            "2011-04-16",
            "2014-04-13",
            "2017-04-09",
            "2021-04-25",
            "2024-04-21",
        )

    def test_ganesh_chaturthi(self):
        name = "Ganesh Chaturthi"
        self.assertNonObservedHolidayName(
            name,
            "2020-08-22",
            "2021-09-10",
            "2022-08-31",
            "2023-09-19",
            "2024-09-07",
            "2025-08-27",
        )
        self.assertNonObservedHolidayName(name, range(self.start_year, 2036))
        self.assertNoHoliday(
            "2010-09-11",
            "2020-08-22",
            "2024-09-07",
        )

    def test_dussehra(self):
        name = "Dussehra"
        self.assertNonObservedHolidayName(
            name,
            "2020-10-25",
            "2021-10-15",
            "2022-10-05",
            "2023-10-24",
            "2024-10-12",
            "2025-10-02",
        )
        self.assertNonObservedHolidayName(name, range(self.start_year, 2036))
        self.assertNoHoliday(
            "2010-10-17",
            "2013-10-13",
            "2017-09-30",
            "2020-10-25",
            "2024-10-12",
        )

    def test_diwali_laxmi_pujan(self):
        name = "Diwali Laxmi Pujan"
        self.assertNonObservedHolidayName(
            name,
            "2020-11-14",
            "2021-11-04",
            "2022-10-24",
            "2023-11-12",
            "2024-11-01",
            "2025-10-21",
        )
        self.assertNonObservedHolidayName(name, range(self.start_year, 2036))
        self.assertNoHoliday(
            "2013-11-03",
            "2016-10-30",
            "2019-10-27",
            "2020-11-14",
            "2023-11-12",
        )

    def test_diwali_balipratipada(self):
        name = "Diwali Balipratipada"
        self.assertNonObservedHolidayName(
            name,
            "2020-11-16",
            "2021-11-05",
            "2022-10-26",
            "2023-11-14",
            "2024-11-02",
            "2025-10-22",
        )
        self.assertNonObservedHolidayName(name, range(self.start_year, 2003), range(2011, 2035))
        self.assertNoNonObservedHolidayName(name, range(2003, 2011))
        self.assertNoHoliday(
            "2024-11-02",
        )

    def test_guru_nanak_jayanti(self):
        name = "Guru Nanak Jayanti"
        self.assertNonObservedHolidayName(
            name,
            "2020-11-30",
            "2021-11-19",
            "2022-11-08",
            "2023-11-27",
            "2024-11-15",
            "2025-11-05",
        )
        self.assertNonObservedHolidayName(name, range(self.start_year, 2036))
        self.assertNoHoliday(
            "2010-11-21",
            "2013-11-17",
            "2017-11-04",
        )

    def test_bhai_dooj(self):
        name = "Bhau Bhij"
        self.assertNonObservedHolidayName(
            name,
            "2003-10-26",
            "2004-11-14",
            "2005-11-03",
            "2006-10-24",
            "2007-11-11",
            "2008-10-30",
            "2009-10-19",
            "2010-11-07",
        )
        self.assertNoNonObservedHolidayName(
            name, range(self.start_year, 2003), range(2011, self.end_year)
        )
        self.assertNoHoliday(
            "2003-10-26",
            "2004-11-14",
            "2007-11-11",
            "2010-11-07",
        )

    def test_buddha_purnima(self):
        name = "Buddha Purnima"
        self.assertNonObservedHolidayName(
            name,
            "2006-05-13",
            "2007-05-02",
            "2008-05-19",
            "2009-05-08",
        )
        self.assertNoNonObservedHolidayName(
            name, range(self.start_year, 2006), range(2010, self.end_year)
        )
        self.assertNoHoliday(
            "2006-05-13",
        )

    def test_ashura(self):
        name = "Muharram"
        self.assertNonObservedHolidayName(
            name,
            "2020-08-30",
            "2021-08-19",
            "2022-08-09",
            "2023-07-29",
            "2024-07-17",
            "2025-07-06",
        )
        self.assertIslamicNoEstimatedNonObservedHolidayName(name, self.full_range)
        self.assertNoHoliday(
            "2012-11-25",
            "2015-10-24",
            "2017-10-01",
            "2020-08-30",
            "2023-07-29",
            "2025-07-06",
        )

    def test_prophets_birthday(self):
        name = "Id-E-Milad-Un-Nabi"
        self.assertNonObservedHolidayName(
            name,
            "2006-04-11",
            "2007-04-01",
            "2008-03-20",
            "2009-03-10",
        )
        self.assertNoIslamicNoEstimatedNonObservedHolidayName(
            name, range(self.start_year, 2006), range(2010, self.end_year)
        )
        self.assertNoHoliday(
            "2007-04-01",
        )

    def test_eid_al_fitr(self):
        name = "Id-Ul-Fitr (Ramadan Eid)"
        self.assertNonObservedHolidayName(
            name,
            "2020-05-25",
            "2021-05-13",
            "2022-05-03",
            "2023-04-22",
            "2024-04-11",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedNonObservedHolidayName(name, self.full_range)
        self.assertNoHoliday(
            "2015-07-18",
            "2018-06-16",
            "2023-04-22",
        )

    def test_eid_al_adha(self):
        name = "Bakri Id"
        self.assertNonObservedHolidayName(
            name,
            "2020-08-01",
            "2021-07-21",
            "2022-07-10",
            "2023-06-28",
            "2024-06-17",
            "2025-06-07",
        )
        self.assertIslamicNoEstimatedNonObservedHolidayName(name, self.full_range)
        self.assertNoHoliday(
            "2012-10-27",
            "2017-09-02",
            "2020-08-01",
            "2022-07-10",
            "2025-06-07",
        )

    def test_2024(self):
        self.assertHolidays(
            NationalStockExchangeOfIndia(years=2024),
            ("2024-01-26", "Republic Day"),
            ("2024-03-08", "Maha Shivaratri"),
            ("2024-03-25", "Holi"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-11", "Id-Ul-Fitr (Ramadan Eid)"),
            ("2024-04-17", "Ram Navami"),
            ("2024-05-01", "Maharashtra Day"),
            ("2024-06-17", "Bakri Id"),
            ("2024-07-17", "Muharram"),
            ("2024-08-15", "Independence Day"),
            ("2024-10-02", "Mahatma Gandhi Jayanti"),
            ("2024-11-01", "Diwali Laxmi Pujan"),
            ("2024-11-15", "Guru Nanak Jayanti"),
            ("2024-12-25", "Christmas Day"),
        )

    def test_2025(self):
        self.assertHolidays(
            NationalStockExchangeOfIndia(years=2025),
            ("2025-02-26", "Maha Shivaratri"),
            ("2025-03-14", "Holi"),
            ("2025-03-31", "Id-Ul-Fitr (Ramadan Eid)"),
            ("2025-04-10", "Mahavir Jayanti"),
            ("2025-04-14", "Dr. Baba Saheb Ambedkar Jayanti"),
            ("2025-04-18", "Good Friday"),
            ("2025-05-01", "Maharashtra Day"),
            ("2025-08-15", "Independence Day"),
            ("2025-08-27", "Ganesh Chaturthi"),
            ("2025-10-02", "Dussehra; Mahatma Gandhi Jayanti"),
            ("2025-10-21", "Diwali Laxmi Pujan"),
            ("2025-10-22", "Diwali Balipratipada"),
            ("2025-11-05", "Guru Nanak Jayanti"),
            ("2025-12-25", "Christmas Day"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-26", "Republic Day"),
            ("2023-03-07", "Holi"),
            ("2023-03-30", "Ram Navami"),
            ("2023-04-04", "Mahavir Jayanti"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-14", "Dr. Baba Saheb Ambedkar Jayanti"),
            ("2023-05-01", "Maharashtra Day"),
            ("2023-06-28", "Bakri Id"),
            ("2023-08-15", "Independence Day"),
            ("2023-09-19", "Ganesh Chaturthi"),
            ("2023-10-02", "Mahatma Gandhi Jayanti"),
            ("2023-10-24", "Dussehra"),
            ("2023-11-14", "Diwali Balipratipada"),
            ("2023-11-27", "Guru Nanak Jayanti"),
            ("2023-12-25", "Christmas Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-26", "Republic Day"),
            ("2023-03-07", "Holi"),
            ("2023-03-30", "Ram Navami"),
            ("2023-04-04", "Mahavir Jayanti"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-14", "Dr. B. R. Ambedkar Jayanti"),
            ("2023-05-01", "Maharashtra Day"),
            ("2023-06-28", "Eid al-Adha"),
            ("2023-08-15", "Independence Day"),
            ("2023-09-19", "Ganesh Chaturthi"),
            ("2023-10-02", "Gandhi Jayanti"),
            ("2023-10-24", "Dussehra"),
            ("2023-11-14", "Diwali Balipratipada"),
            ("2023-11-27", "Guru Nanak Jayanti"),
            ("2023-12-25", "Christmas Day"),
        )

    def test_l10n_gu(self):
        self.assertLocalizedHolidays(
            "gu",
            ("2023-01-26", "પ્રજાસત્તાક દિવસ"),
            ("2023-03-07", "હોળી"),
            ("2023-03-30", "રામ નવમી"),
            ("2023-04-04", "મહાવીર જયંતિ"),
            ("2023-04-07", "ગુડ ફ્રાઈડે"),
            ("2023-04-14", "ડૉ. બાબાસાહેબ આંબેડકર જયંતિ"),
            ("2023-05-01", "મહારાષ્ટ્ર દિવસ"),
            ("2023-06-28", "બકરી ઈદ"),
            ("2023-08-15", "સ્વતંત્રતા દિવસ"),
            ("2023-09-19", "ગણેશ ચતુર્થી"),
            ("2023-10-02", "મહાત્મા ગાંધી જયંતિ"),
            ("2023-10-24", "દશેરા"),
            ("2023-11-14", "દિવાળી બલિપ્રતિપદા"),
            ("2023-11-27", "ગુરુ નાનક જયંતિ"),
            ("2023-12-25", "નાતાલ"),
        )

    def test_l10n_hi(self):
        self.assertLocalizedHolidays(
            "hi",
            ("2023-01-26", "गणतंत्र दिवस"),
            ("2023-03-07", "होली"),
            ("2023-03-30", "राम नवमी"),
            ("2023-04-04", "महावीर जयंती"),
            ("2023-04-07", "गुड फ्राइडे"),
            ("2023-04-14", "डॉ. बाबा साहेब अम्बेडकर जयंती"),
            ("2023-05-01", "महाराष्ट्र दिवस"),
            ("2023-06-28", "बकरीद"),
            ("2023-08-15", "स्वतंत्रता दिवस"),
            ("2023-09-19", "गणेश चतुर्थी"),
            ("2023-10-02", "महात्मा गांधी जयंती"),
            ("2023-10-24", "दशहरा"),
            ("2023-11-14", "दिवाली बलिप्रतिपदा"),
            ("2023-11-27", "गुरु नानक जयंती"),
            ("2023-12-25", "क्रिसमस डे"),
        )

    def test_l10n_mr(self):
        self.assertLocalizedHolidays(
            "mr",
            ("2023-01-26", "प्रजासत्ताक दिन"),
            ("2023-03-07", "होळी"),
            ("2023-03-30", "रामनवमी"),
            ("2023-04-04", "महावीर जन्म कल्याणक"),
            ("2023-04-07", "गुड फ्रायडे"),
            ("2023-04-14", "डॉ. बाबासाहेब आंबेडकर जयंती"),
            ("2023-05-01", "महाराष्ट्र दिन"),
            ("2023-06-28", "बकरी ईद"),
            ("2023-08-15", "स्वातंत्र्य दिन"),
            ("2023-09-19", "गणेश चतुर्थी"),
            ("2023-10-02", "महात्मा गांधी जयंती"),
            ("2023-10-24", "दसरा"),
            ("2023-11-14", "दिवाळी बलिप्रतिपदा"),
            ("2023-11-27", "गुरुनानक जयंती"),
            ("2023-12-25", "नाताळ"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.denmark import Denmark
from tests.common import CommonCountryTests


class TestDenmark(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Denmark)

    def test_new_years_day(self):
        self.assertHolidayName("Nytårsdag", (f"{year}-01-01" for year in self.full_range))

    def test_maundy_thursday(self):
        name = "Skærtorsdag"
        self.assertHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Langfredag"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Påskedag"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Anden påskedag"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_workers_day(self):
        name = "Arbejdernes kampdag"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name, (f"{year}-05-01" for year in range(1890, self.end_year))
        )
        self.assertNoOptionalHolidayName(name, range(self.start_year, 1890))

    def test_great_day_of_prayers(self):
        name = "Store bededag"
        self.assertHolidayName(
            name,
            "2020-05-08",
            "2021-04-30",
            "2022-05-13",
            "2023-05-05",
        )
        self.assertHolidayName(name, range(self.start_year, 2024))
        self.assertNoHolidayName(name, range(2024, self.end_year))

    def test_ascension_day(self):
        name = "Kristi himmelfartsdag"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_sunday(self):
        name = "Pinsedag"
        self.assertHolidayName(
            name,
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
            "2024-05-19",
            "2025-06-08",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Anden pinsedag"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_constitution_day(self):
        name = "Grundlovsdag"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name, (f"{year}-06-05" for year in range(1891, self.end_year))
        )
        self.assertNoOptionalHolidayName(name, range(self.start_year, 1891))

    def test_christmas_eve(self):
        name = "Juleaftensdag"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(name, (f"{year}-12-24" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Juledag", (f"{year}-12-25" for year in self.full_range))

    def test_second_day_of_christmas(self):
        self.assertHolidayName("Anden juledag", (f"{year}-12-26" for year in self.full_range))

    def test_new_years_eve(self):
        name = "Nytårsaften"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(name, (f"{year}-12-31" for year in self.full_range))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Nytårsdag"),
            ("2022-04-14", "Skærtorsdag"),
            ("2022-04-15", "Langfredag"),
            ("2022-04-17", "Påskedag"),
            ("2022-04-18", "Anden påskedag"),
            ("2022-05-01", "Arbejdernes kampdag"),
            ("2022-05-13", "Store bededag"),
            ("2022-05-26", "Kristi himmelfartsdag"),
            ("2022-06-05", "Grundlovsdag; Pinsedag"),
            ("2022-06-06", "Anden pinsedag"),
            ("2022-12-24", "Juleaftensdag"),
            ("2022-12-25", "Juledag"),
            ("2022-12-26", "Anden juledag"),
            ("2022-12-31", "Nytårsaften"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-14", "Maundy Thursday"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Workers' Day"),
            ("2022-05-13", "Great Prayer Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-05", "Constitution Day; Whit Sunday"),
            ("2022-06-06", "Whit Monday"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-04-14", "วันพฤหัสศักดิ์สิทธิ์"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-17", "วันอาทิตย์อีสเตอร์"),
            ("2022-04-18", "วันจันทร์อีสเตอร์"),
            ("2022-05-01", "วันแรงงาน"),
            ("2022-05-13", "วันแห่งการอธิษฐานใหญ่"),
            ("2022-05-26", "วันสมโภชพระเยซูเจ้าเสด็จขึ้นสวรรค์"),
            ("2022-06-05", "วันรัฐธรรมนูญ; วันสมโภชพระจิตเจ้า"),
            ("2022-06-06", "วันจันทร์หลังวันสมโภชพระจิตเจ้า"),
            ("2022-12-24", "วันคริสต์มาสอีฟ"),
            ("2022-12-25", "วันคริสต์มาส"),
            ("2022-12-26", "วันคริสต์มาสวันที่สอง"),
            ("2022-12-31", "วันสิ้นปี"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-04-14", "Великий четвер"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День трудящих"),
            ("2022-05-13", "День загальної молитви"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-06-05", "День Конституції; Трійця"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
            ("2022-12-31", "Переддень Нового року"),
        )



#!/usr/bin/env python3


#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

"""Generate Gregorian dates for holidays based on the Hebrew lunisolar calendar.

Run with:

    python -m scripts.calendar.hebrew_generator

Alternatively, run with uv:

    uv run -m scripts.calendar.hebrew_generator

This generates the file `holidays/calendars/hebrew_dates.py`,
whose data can then be copied to `holidays/calendars/hebrew.py`.
"""

from collections import defaultdict
from datetime import date
from functools import cache

from .generator import CalendarGenerator

HEBREW_HOLIDAYS = {
    "PASSOVER": (1, 15),
    "INDEPENDENCE_DAY": (2, 5),
    "LAG_BAOMER": (2, 18),
    "SHAVUOT": (3, 6),
    "TISHA_BAV": (5, 9),
    "ROSH_HASHANAH": (7, 1),
    "YOM_KIPPUR": (7, 10),
    "SUKKOT": (7, 15),
    "HANUKKAH": (9, 25),
    "PURIM": (12, 14),
}

HEBREW_EPOCH_RD = -1373427  # 347997 [Hebrew epoch JD] - 1721425 [Julian date of 1 Jan 1 CE] + 1.
HEBREW_YEAR_OFFSET = 3760  # Offset to convert Gregorian year to Hebrew year.


class _Lunisolar:
    """Convert dates from the Hebrew lunisolar calendar to Gregorian dates.

    References:
        * <https://en.wikipedia.org/wiki/Hebrew_calendar>
    """

    @staticmethod
    def leap(year: int) -> bool:
        """Return True if a Hebrew year is a leap year."""
        return ((year * 7) + 1) % 19 < 7

    @staticmethod
    @cache
    def elapsed_days(year: int) -> int:
        months = ((235 * year) - 234) // 19
        parts = 12084 + 13753 * months
        day = months * 29 + parts // 25920

        # A year cannot start on a Sunday, Wednesday, or Friday.
        if (3 * (day + 1)) % 7 < 3:
            day += 1

        return day

    def year_length_correction(self, year: int) -> int:
        present_year_len = self.elapsed_days(year)
        next_year_len = self.elapsed_days(year + 1)
        if next_year_len - present_year_len == 356:
            return 2

        last_year_len = self.elapsed_days(year - 1)
        if present_year_len - last_year_len == 382:
            return 1

        return 0

    def new_year(self, year: int) -> int:
        return HEBREW_EPOCH_RD + self.elapsed_days(year) + self.year_length_correction(year)

    @cache
    def days_in_year(self, year: int) -> int:
        return self.new_year(year + 1) - self.new_year(year)

    @cache
    def month_offsets(self, year: int) -> list[int]:
        """Cumulative day offsets for months in a given Hebrew year."""
        is_leap = self.leap(year)
        months_in_year = 13 if is_leap else 12
        year_len_mod = self.days_in_year(year) % 10

        offsets = [0] * (months_in_year + 1)
        days = 0
        for m in range(1, months_in_year + 1):
            offsets[m] = days

            match m:
                # Fixed-length 29 day months: Iyyar, Tammuz, Elul, Tevet, Adar II.
                case 2 | 4 | 6 | 10 | 13:
                    days += 29
                # Short Marcheshvan (days depend on length of year).
                case 8 if year_len_mod != 5:
                    days += 29
                # Short Kislev (the same).
                case 9 if year_len_mod == 3:
                    days += 29
                # Short Adar (in non-leap years).
                case 12 if not is_leap:
                    days += 29
                case _:
                    days += 30

        return offsets

    def to_rd(self, year: int, month: int, day: int) -> int:
        """Convert a Hebrew date to a Rata Die (RD) day count."""
        offsets = self.month_offsets(year)
        delta = offsets[month] - offsets[7]
        if month < 7:
            delta += self.days_in_year(year)

        return self.new_year(year) + delta + day - 1


def generate_data() -> None:
    cal = _Lunisolar()
    g_year_min, g_year_max = 1947, 2100

    dates: dict[str, dict[int, date]] = defaultdict(dict)
    for name, (holiday_month, holiday_day) in HEBREW_HOLIDAYS.items():
        for year in range(g_year_min + HEBREW_YEAR_OFFSET, g_year_max + HEBREW_YEAR_OFFSET + 2):
            g_date = date.fromordinal(
                cal.to_rd(
                    year,
                    holiday_month + 1 if holiday_month == 12 and cal.leap(year) else holiday_month,
                    holiday_day,
                )
            )
            g_year = g_date.year
            if g_year < g_year_min or g_year > g_year_max:
                continue
            dates[name][g_year] = g_date

    cal_gen = CalendarGenerator("hebrew", "_HebrewLunisolar")
    cal_gen.generate(dates)


if __name__ == "__main__":
    generate_data()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.belize import Belize
from tests.common import CommonCountryTests


class TestBelize(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Belize)

    def test_new_years_day(self):
        name = "New Year's Day"
        years_observed = {1984, 1989, 1995, 2006, 2012, 2017, 2023, 2034, 2040, 2045}
        self.assertHolidayName(
            name, (f"{year}-01-01" for year in self.full_range if year not in years_observed)
        )
        self.assertHolidayName(f"{name} (observed)", (f"{year}-01-02" for year in years_observed))

    def test_george_price_day(self):
        name = "George Price Day"
        years_observed = {2023, 2034, 2040, 2045}
        self.assertHolidayName(
            name,
            (f"{year}-01-15" for year in range(2021, self.end_year) if year not in years_observed),
        )
        self.assertHolidayName(f"{name} (observed)", (f"{year}-01-16" for year in years_observed))
        self.assertNoHolidayName(name, range(self.start_year, 2021))

    def test_national_heroes_and_benefactors_day(self):
        name = "National Heroes and Benefactors Day"
        self.assertHolidayName(
            f"{name} (observed)",
            "1982-03-08",
            "1983-03-07",
            "1984-03-12",
            "1993-03-08",
            "2000-03-06",
            "2010-03-08",
            "2018-03-12",
            "2021-03-08",
            "2022-03-07",
            "2023-03-06",
        )
        self.assertNonObservedHolidayName(name, (f"{year}-03-09" for year in self.full_range))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_holy_saturday(self):
        name = "Holy Saturday"
        self.assertHolidayName(
            name,
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            "2023-04-08",
            "2024-03-30",
            "2025-04-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Labour Day"
        years_observed = {1983, 1988, 1994, 2005, 2011, 2016, 2022, 2033, 2039, 2044}
        self.assertHolidayName(
            name, (f"{year}-05-01" for year in self.full_range if year not in years_observed)
        )
        self.assertHolidayName(f"{name} (observed)", (f"{year}-05-02" for year in years_observed))

    def test_commonwealth_day(self):
        name = "Commonwealth Day"
        self.assertHolidayName(
            f"{name} (observed)",
            "1983-05-23",
            "1984-05-21",
            "1992-05-25",
            "2000-05-22",
            "2011-05-23",
            "2018-05-21",
            "2019-05-27",
            "2020-05-25",
        )
        self.assertNonObservedHolidayName(
            name, (f"{year}-05-24" for year in range(self.start_year, 2022))
        )
        self.assertNoHolidayName(name, range(2022, self.end_year))

    def test_emancipation_day(self):
        name = "Emancipation Day"
        self.assertHolidayName(
            f"{name} (observed)",
            "2021-08-02",
            "2023-07-31",
            "2024-07-29",
            "2025-08-04",
        )
        self.assertNonObservedHolidayName(
            name, (f"{year}-08-01" for year in range(2021, self.end_year))
        )
        self.assertNoHolidayName(name, range(self.start_year, 2021))

    def test_saint_georges_caye_day(self):
        name = "Saint George's Caye Day"
        years_observed = {1989, 1995, 2000, 2006, 2017, 2023, 2028, 2034, 2045}
        self.assertHolidayName(
            name, (f"{year}-09-10" for year in self.full_range if year not in years_observed)
        )
        self.assertHolidayName(f"{name} (observed)", (f"{year}-09-11" for year in years_observed))

    def test_independence_day(self):
        name = "Independence Day"
        years_observed = {1986, 1997, 2003, 2008, 2014, 2025, 2031, 2036, 2042}
        self.assertHolidayName(
            name, (f"{year}-09-21" for year in self.full_range if year not in years_observed)
        )
        self.assertHolidayName(f"{name} (observed)", (f"{year}-09-22" for year in years_observed))

    def test_indigenous_peoples_resistance_day(self):
        name1 = "Pan American Day"
        name2 = "Indigenous Peoples' Resistance Day"
        self.assertHolidayName(
            f"{name1} (observed)",
            "1982-10-11",
            "1983-10-10",
            "1993-10-11",
            "2000-10-09",
            "2011-10-10",
            "2018-10-15",
        )
        self.assertHolidayName(
            f"{name2} (observed)",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
            "2025-10-13",
            "2027-10-11",
        )
        self.assertNonObservedHolidayName(
            name1, (f"{year}-10-12" for year in range(self.start_year, 2021))
        )
        self.assertNonObservedHolidayName(
            name2, (f"{year}-10-12" for year in range(2021, self.end_year))
        )
        self.assertNoHolidayName(name1, range(2021, self.end_year))
        self.assertNoHolidayName(name2, range(self.start_year, 2021))

    def test_garifuna_settlement_day(self):
        name = "Garifuna Settlement Day"
        years_observed = {1989, 1995, 2000, 2006, 2017, 2023, 2028, 2034, 2045}
        self.assertHolidayName(
            name, (f"{year}-11-19" for year in self.full_range if year not in years_observed)
        )
        self.assertHolidayName(f"{name} (observed)", (f"{year}-11-20" for year in years_observed))

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in self.full_range))

    def test_boxing_day(self):
        name = "Boxing Day"
        years_observed = {1982, 1993, 1999, 2004, 2010, 2021, 2027, 2032, 2038, 2049}
        self.assertHolidayName(
            name, (f"{year}-12-26" for year in self.full_range if year not in years_observed)
        )
        self.assertHolidayName(f"{name} (observed)", (f"{year}-12-27" for year in years_observed))

    def test_2021(self):
        # https://www.pressoffice.gov.bz/public-and-bank-holidays-2021-3/
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "New Year's Day"),
            ("2021-01-15", "George Price Day"),
            ("2021-03-08", "National Heroes and Benefactors Day (observed)"),
            ("2021-04-02", "Good Friday"),
            ("2021-04-03", "Holy Saturday"),
            ("2021-04-05", "Easter Monday"),
            ("2021-05-01", "Labour Day"),
            ("2021-05-24", "Commonwealth Day"),
            ("2021-08-02", "Emancipation Day (observed)"),
            ("2021-09-10", "Saint George's Caye Day"),
            ("2021-09-21", "Independence Day"),
            ("2021-10-11", "Indigenous Peoples' Resistance Day (observed)"),
            ("2021-11-19", "Garifuna Settlement Day"),
            ("2021-12-25", "Christmas Day"),
            ("2021-12-27", "Boxing Day (observed)"),
        )

    def test_2022(self):
        # https://www.pressoffice.gov.bz/public-and-bank-holidays-2022-updated/
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-01-15", "George Price Day"),
            ("2022-03-07", "National Heroes and Benefactors Day (observed)"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-16", "Holy Saturday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-02", "Labour Day (observed)"),
            ("2022-08-01", "Emancipation Day"),
            ("2022-09-10", "Saint George's Caye Day"),
            ("2022-09-21", "Independence Day"),
            ("2022-10-10", "Indigenous Peoples' Resistance Day (observed)"),
            ("2022-11-19", "Garifuna Settlement Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
        )
